@@ -55,15 +55,18 @@ MBArray * mbarray_Create(BTYPE num_bits, const char * file, const char * header,
     array->preamblesize = ((int)ceil((double)array->preamblebytes / 256.0) * 256) / sizeof(DTYPE);
     array->preamblebytes = array->preamblesize * (sizeof(DTYPE));
 
-    array->size = (int)ceil((double)num_bits / sizeof(DTYPE) / 8.0);
-    array->bytes = (int)ceil((double)num_bits / 8.0);
-
     if (errno) {
         mbarray_Destroy(array);
         return NULL;
     }
 
     filesize = _filesize(array->fd);
+    if (filesize > 50 && !num_bits) {
+        num_bits = _get_num_bits(array->fd);
+    }
+    array->size = (int)ceil((double)num_bits / sizeof(DTYPE) / 8.0);
+    array->bytes = (int)ceil((double)num_bits / 8.0);
+
     if (filesize < 0) {
         mbarray_Destroy(array);
         return NULL;
@@ -73,6 +76,11 @@ MBArray * mbarray_Create(BTYPE num_bits, const char * file, const char * header,
         mbarray_Destroy(array);
         return NULL;
     }
+    else if (filesize && filesize < (array->bytes + array->preamblebytes - 1)) {
+        errno = EINVAL;
+        mbarray_Destroy(array);
+        return NULL;
+    }
     else if (!filesize) {
         if (!(oflag & O_CREAT) || (!num_bits) || _initialize_file(array->fd, array->bytes + array->preamblebytes - 1, num_bits, header, header_len)) {
             if (!errno) {
@@ -118,14 +126,16 @@ MBArray * mbarray_Create(BTYPE num_bits, const char * file, const char * header,
 
 void mbarray_Destroy(MBArray * array)
 {
-    if (array) {
-        if (array->vector) {
+    if (array != NULL) {
+        if (array->vector != NULL) {
             if (munmap(array->vector, _mmap_size(array))) {
                 fprintf(stderr, "Unable to close mmap!\n");
             }
+            array->vector = NULL;
         }
         if (array->filename) {
             free((void *)array->filename);
+            array->filename = NULL;
         }
         free(array);
     }
